import { useCallback, useMemo } from "react";
import { DataType } from "@kie-tools/boxed-expression-component/dist/api";
import { CELL_MINIMUM_WIDTH } from "@kie-tools/unitables/dist/boxed";
export function isOutputWithInsideProperties(toBeDetermined) {
    return toBeDetermined.insideProperties !== undefined;
}
export function useDmnBoxedOutputs(jsonSchemaBridge, results, rowCount, outputColumnsCache) {
    const updateOutputCellsWidth = useCallback((outputs) => {
        var _a;
        (_a = outputColumnsCache.current) === null || _a === void 0 ? void 0 : _a.forEach((cachedColumn) => {
            if (cachedColumn.groupType === "output") {
                const outputToUpdate = outputs.find((e) => e.name === cachedColumn.label);
                if (outputToUpdate && isOutputWithInsideProperties(outputToUpdate) && (cachedColumn === null || cachedColumn === void 0 ? void 0 : cachedColumn.columns)) {
                    outputToUpdate.insideProperties.forEach((insideProperty) => {
                        var _a;
                        if (insideProperty !== null && typeof insideProperty === "object") {
                            Object.keys(insideProperty).map((insidePropertyKey) => {
                                var _a;
                                const columnFound = (_a = cachedColumn.columns) === null || _a === void 0 ? void 0 : _a.find((nestedColumn) => nestedColumn.label === insidePropertyKey);
                                if (columnFound) {
                                    insideProperty[insidePropertyKey] = {
                                        value: insideProperty[insidePropertyKey],
                                        width: columnFound.width,
                                    };
                                }
                            });
                        }
                        else if (insideProperty) {
                            const columnFound = (_a = cachedColumn.columns) === null || _a === void 0 ? void 0 : _a.find((nestedColumn) => nestedColumn.label === insideProperty.name);
                            if (columnFound) {
                                insideProperty.width = columnFound.width;
                            }
                        }
                    });
                }
                if (outputToUpdate) {
                    outputToUpdate.width = cachedColumn.width;
                }
            }
        });
    }, [outputColumnsCache]);
    const deepFlattenOutput = useCallback((acc, entry, value) => {
        return Object.entries(value).map(([deepEntry, deepValue]) => {
            if (typeof deepValue === "object" && deepValue !== null) {
                deepFlattenOutput(acc, deepEntry, deepValue);
            }
            acc[`${entry}-${deepEntry}`] = deepValue;
            return acc;
        });
    }, []);
    const deepGenerateOutputTypesMapFields = useCallback((outputTypeMap, properties, jsonSchemaBridge) => {
        return Object.entries(properties).map(([name, property]) => {
            if (property["x-dmn-type"]) {
                const dataType = jsonSchemaBridge.getBoxedDataType(property).dataType;
                outputTypeMap.set(name, { type: property.type, dataType, name });
                return { name, type: property.type, width: CELL_MINIMUM_WIDTH, dataType };
            }
            const path = property.$ref.split("/").slice(1);
            const data = path.reduce((acc, property) => acc[property], jsonSchemaBridge.schema);
            const dataType = jsonSchemaBridge.getBoxedDataType(data).dataType;
            if (data.properties) {
                const insideProperties = deepGenerateOutputTypesMapFields(outputTypeMap, data.properties, jsonSchemaBridge);
                outputTypeMap.set(name, { type: data.type, insideProperties, dataType, name });
            }
            else {
                outputTypeMap.set(name, { type: data.type, dataType, name });
            }
            return { name, dataType: data.type, width: CELL_MINIMUM_WIDTH };
        });
    }, []);
    const { outputs, outputRules } = useMemo(() => {
        var _a, _b, _c, _d;
        const decisionResults = results === null || results === void 0 ? void 0 : results.filter((result) => result !== undefined);
        if (jsonSchemaBridge === undefined || decisionResults === undefined) {
            return { outputs: [], outputRules: [] };
        }
        const outputTypeMap = Object.entries((_d = (_c = (_b = (_a = jsonSchemaBridge.schema) === null || _a === void 0 ? void 0 : _a.definitions) === null || _b === void 0 ? void 0 : _b.OutputSet) === null || _c === void 0 ? void 0 : _c.properties) !== null && _d !== void 0 ? _d : []).reduce((outputTypeMap, [name, properties]) => {
            if (properties["x-dmn-type"]) {
                const dataType = jsonSchemaBridge.getBoxedDataType(properties).dataType;
                outputTypeMap.set(name, { type: properties.type, dataType, name });
            }
            else {
                const path = properties.$ref.split("/").slice(1);
                const data = path.reduce((acc, property) => acc[property], jsonSchemaBridge.schema);
                const dataType = jsonSchemaBridge.getBoxedDataType(data).dataType;
                if (data.properties) {
                    const insideProperties = deepGenerateOutputTypesMapFields(outputTypeMap, data.properties, jsonSchemaBridge);
                    outputTypeMap.set(name, { type: data.type, insideProperties, dataType, name });
                }
                else {
                    outputTypeMap.set(name, { type: data.type, dataType, name });
                }
            }
            return outputTypeMap;
        }, new Map());
        const outputMap = decisionResults.reduce((acc, decisionResult) => {
            if (decisionResult) {
                decisionResult.forEach(({ decisionName }) => {
                    var _a;
                    const data = outputTypeMap.get(decisionName);
                    const dataType = (_a = data === null || data === void 0 ? void 0 : data.dataType) !== null && _a !== void 0 ? _a : DataType.Undefined;
                    if (data && isOutputWithInsideProperties(data)) {
                        acc.set(decisionName, {
                            name: decisionName,
                            dataType,
                            insideProperties: data.insideProperties,
                            width: data.insideProperties.reduce((acc, column) => acc + column.width, 0),
                        });
                    }
                    else {
                        acc.set(decisionName, {
                            name: decisionName,
                            dataType,
                            width: CELL_MINIMUM_WIDTH,
                        });
                    }
                });
            }
            return acc;
        }, new Map());
        const outputEntries = decisionResults.reduce((acc, decisionResult) => {
            if (decisionResult) {
                const outputResults = decisionResult.map(({ result, decisionName }) => {
                    if (result === null || typeof result !== "object") {
                        const dmnRunnerClause = outputMap.get(decisionName);
                        if (dmnRunnerClause && isOutputWithInsideProperties(dmnRunnerClause)) {
                            return dmnRunnerClause.insideProperties.reduce((acc, insideProperty) => {
                                acc[insideProperty.name] = "null";
                                return acc;
                            }, {});
                        }
                    }
                    if (result === null) {
                        return "null";
                    }
                    if (result === true) {
                        return "true";
                    }
                    if (result === false) {
                        return "false";
                    }
                    if (typeof result === "object") {
                        return Object.entries(result).reduce((flattenObject, [entry, value]) => {
                            if (typeof value === "object" && value !== null) {
                                deepFlattenOutput(flattenObject, entry, value);
                            }
                            else {
                                flattenObject[entry] = value;
                            }
                            return flattenObject;
                        }, {});
                    }
                    return result;
                });
                return [...acc, outputResults];
            }
            return acc;
        }, []);
        const outputRules = Array.from(Array(rowCount)).map((e, i) => {
            var _a;
            return ({
                outputEntries: (_a = outputEntries === null || outputEntries === void 0 ? void 0 : outputEntries[i]) !== null && _a !== void 0 ? _a : [],
            });
        });
        const outputs = Array.from(outputMap.values());
        updateOutputCellsWidth(outputs);
        return { outputs, outputRules };
    }, [
        deepFlattenOutput,
        deepGenerateOutputTypesMapFields,
        jsonSchemaBridge,
        results,
        rowCount,
        updateOutputCellsWidth,
    ]);
    return useMemo(() => {
        return {
            outputs,
            outputRules,
            updateOutputCellsWidth,
        };
    }, [outputRules, outputs, updateOutputCellsWidth]);
}
//# sourceMappingURL=DmnBoxedOutputs.js.map