import Ajv from "ajv";
import metaSchemaDraft04 from "ajv/lib/refs/json-schema-draft-04.json";
import { DmnUnitablesJsonSchemaBridge } from "./uniforms/DmnUnitablesJsonSchemaBridge";
import { DAYS_AND_TIME_DURATION_FORMAT, YEARS_AND_MONTHS_DURATION_FORMAT } from "@kie-tools/form-dmn/dist/uniforms";
import { UnitablesValidator } from "@kie-tools/unitables/dist/UnitablesValidator";
const DAYS_AND_TIME = /^(-|\+)?P(?:([-+]?[0-9]*)D)?(?:T(?:([-+]?[0-9]*)H)?(?:([-+]?[0-9]*)M)?(?:([-+]?[0-9]*)S)?)?$/;
const YEARS_AND_MONTHS = /^(-|\+)?P(?:([-+]?[0-9]*)Y)?(?:([-+]?[0-9]*)M)?$/;
export class DmnUnitablesValidator extends UnitablesValidator {
    constructor(i18n) {
        super(i18n);
        this.ajv = new Ajv({ allErrors: true, schemaId: "auto", useDefaults: true });
        this.SCHEMA_DRAFT4 = "http://json-schema.org/draft-04/schema#";
        this.setupValidator();
    }
    setupValidator() {
        this.ajv.addMetaSchema(metaSchemaDraft04);
        this.ajv.addFormat(DAYS_AND_TIME_DURATION_FORMAT, {
            type: "string",
            validate: (data) => !!data.match(DAYS_AND_TIME),
        });
        this.ajv.addFormat(YEARS_AND_MONTHS_DURATION_FORMAT, {
            type: "string",
            validate: (data) => !!data.match(YEARS_AND_MONTHS),
        });
    }
    createValidator(jsonSchema) {
        const validator = this.ajv.compile(jsonSchema);
        return (model) => {
            validator(JSON.parse(JSON.stringify(model)));
            if (validator.errors && validator.errors.length) {
                const details = validator.errors
                    .filter((error) => error.keyword !== "required")
                    .map((error) => {
                    if (error.keyword === "format") {
                        if (error.params.format === DAYS_AND_TIME_DURATION_FORMAT) {
                            return Object.assign(Object.assign({}, error), { message: "" });
                        }
                        if (error.params.format === YEARS_AND_MONTHS_DURATION_FORMAT) {
                            return Object.assign(Object.assign({}, error), { message: "" });
                        }
                    }
                });
                return { details };
            }
            return null;
        };
    }
    getBridge(formSchema) {
        const formDraft4 = Object.assign(Object.assign({}, formSchema), { $schema: this.SCHEMA_DRAFT4 });
        const validator = this.createValidator(formDraft4);
        return new DmnUnitablesJsonSchemaBridge(formDraft4, validator, this.i18n);
    }
}
//# sourceMappingURL=DmnUnitablesValidator.js.map